char LicenseText[]=
"LICENSE\n"
" \n"
" Permission is hereby granted, free of charge, to any person obtaining a copy of this software\n"
" and associated documentation files (the \"Software\"), to deal in the Software without restriction,\n"
" including without limitation the rights to use, copy, modify,\n"
" merge, publish, distribute, sublicense, and/or sell copies of the Software,\n"
" and to permit persons to whom the Software is furnished to do so,\n"
" subject to the following conditions:\n"
" \n"
" The above copyright notice and this permission notice shall be included\n"
" in all copies or substantial portions of the Software.\n"
" \n"
" THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,\n"
" INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.\n"
" IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,\n"
" WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\n"
" OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n";


#include <stdint.h>
#include <string>
#include <stdio.h>
#include <iostream>

#ifdef _WIN32
#include "windows.h"

#define __UTF16_CMDLINE__

#endif

template <class T> class SafeArray {
protected:
    T * ArrayItems;
    int64_t ArrayLength;

public:
    SafeArray() : ArrayItems(0), ArrayLength(0) {
    }

    ~SafeArray() {
        if(ArrayItems) delete [] ArrayItems;
        ArrayItems=nullptr;
        ArrayLength=0;
    }

    T * data() {return ArrayItems;}
    int64_t length() {return ArrayLength;}


    bool AssignLength(int64_t len) {
        if(len == 0) return false;
        if(ArrayItems) delete [] ArrayItems;
        ArrayItems=nullptr; ArrayLength=0;
        try {
        ArrayItems=new T[len];
        }
        catch(...) {
        ArrayItems=nullptr;ArrayLength=0;return false;
        }
        ArrayLength=len;
        return ArrayLength;
    }

};


using namespace std;

#ifdef _WIN32
typedef std::wstring IntString;

IntString StrToWCHAR(char * str)
{
    SafeArray <wchar_t> WCHARString;
    size_t StringLen=strlen(str);
    WCHARString.AssignLength(StringLen+1);
    memset((char *)WCHARString.data(), 0, sizeof(wchar_t)*WCHARString.length());
    int Res=MultiByteToWideChar(CP_UTF8, 0, str, StringLen, (wchar_t *)WCHARString.data(), WCHARString.length());
    if(!Res) {
        cout << "error: string encoding " << str << endl;
        return IntString(L"");
        }
    return IntString((wchar_t *)WCHARString.data());
}
#else
typedef std::string IntString;
#endif
inline void SwapShortWord(uint16_t *data) {
uint16_t tmpsour=*data;
char *s=(char *)&tmpsour;
char *d=(char *)data;
d[1]=s[0];
d[0]=s[1];
}

inline void SwapLongWord(uint32_t *data) {
uint32_t tmpsour=*data;
char *s=(char *)&tmpsour;
char *d=(char *)data;
d[3]=s[0];
d[2]=s[1];
d[1]=s[2];
d[0]=s[3];
}

inline void SwapLongLongWord(uint64_t *data) {
uint64_t tmpsour=*data;
char *s=(char *)&tmpsour;
char *d=(char *)data;

d[7]=s[0];
d[6]=s[1];
d[5]=s[2];
d[4]=s[3];
d[3]=s[4];
d[2]=s[5];
d[1]=s[6];
d[0]=s[7];
}

inline void SwapCharBites(char * x) {
char s=*x;
char y=0;
y=(s&0x80)>>7;
y|=(s&0x01)<<7;
y|=(s&0x40)>>5;
y|=(s&0x02)<<5;
y|=(s&0x20)>>3;
y|=(s&0x04)<<3;
y|=(s&0x10)>>1;
y|=(s&0x08)<<1;
*x=y;
}


typedef FILE * FileHandle;

FileHandle OpenFile(IntString FileName, bool Read)
{
FileHandle Handle;
#ifdef _WIN32
if(Read)
    Handle=_wfopen(FileName.c_str(), L"rb");
else
    Handle=_wfopen(FileName.c_str(), L"wb");

#else
if(Read)
    Handle=fopen(FileName.c_str(), "rb");
else
    Handle=fopen(FileName.c_str(), "wb");
#endif
if(!Handle)
    cout << "error: " << strerror(errno) << endl;
return Handle;
}

void CloseFile(FileHandle Handle)
{
fclose(Handle);
}

size_t ReadFile(FileHandle Handle, char * data, size_t len)
{
size_t bytes=fread(data, 1, len, Handle);
return bytes;
}

size_t WriteFile(FileHandle Handle, char * data, size_t len)
{
size_t bytes=fwrite(data, 1, len, Handle);
return bytes;
}


int64_t FileCurrentPosition(FileHandle Handle)
{
return ftello(Handle);
}

void FileGotoPosition(FileHandle Handle, int64_t Position)
{
fseeko(Handle, Position, SEEK_SET);
}

void FileGotoFromCurrent(FileHandle Handle, uint64_t Offset)
{
fseeko(Handle, Offset, SEEK_CUR);
}







void RemoveFile(IntString FileName)
{
#ifdef _WIN32
_wremove(FileName.c_str());
#else
remove(FileName.c_str());
#endif
}

IntString ExtractFileExtension(IntString FileName)
{
#ifdef _WIN32
IntString result=L"";
int FileNameLength=wcslen(FileName.c_str());
#else
IntString result="";
int FileNameLength=strlen(FileName.c_str());
#endif
if(FileNameLength < 4) {
    cout << "wrong file name \"" << FileName.c_str() << "\"" << endl;
    return result;
}
IntString ext=FileName.substr(FileNameLength-4, 4);

return ext;
}




#pragma pack(push)
#pragma pack(1)

struct DSFHeader
{
unsigned char FileID[4]={'D','S','D',' '};
uint64_t HeaderLength;
uint64_t FileLength;
uint64_t MetadataPointer;

char FMTID[4];
uint64_t FMTLength;

uint32_t FormatVersion;
uint32_t FormatID;
uint32_t ChannelType;
uint32_t ChannelNumber;
uint32_t SamplingRate;
uint32_t BitOrder;
uint64_t SampleCount;
uint32_t BlockSizePerChannel;
uint32_t Reserved;

char ChunkDataID[4];
uint64_t DataLength;

static const uint32_t DSFBlockLengthBytes=4096;
static const uint32_t DSFBlockLengthSamples=DSFBlockLengthBytes*8;

DSFHeader() {
    HeaderLength=28;
    FileLength=sizeof(DSFHeader);
    DataLength=12;
    SampleCount=0;
    MetadataPointer=0;

    FMTID[0]='f'; FMTID[1]='m'; FMTID[2]='t'; FMTID[3]=' ';
    FMTLength=52;

    ChunkDataID[0]='d'; ChunkDataID[1]='a'; ChunkDataID[2]='t'; ChunkDataID[3]='a';
    BitOrder=1;

    BlockSizePerChannel=DSFBlockLengthBytes;

    FormatVersion=1;
    FormatID=0;
    Reserved=0;
}
};

struct ChunkHeader {
    char ID[4];
    uint64_t Length;
};

struct DFFChunkFRM8 {
    char DSD_ID[4];
};


struct DFFChunkVersion {
    uint32_t Version;
};

struct DFFChunkSamplingRate {
    uint32_t SamplingRate;
};

struct DFFChunkChannels {
    uint16_t ChannelNumber;
};

struct DFFChunkCompression {
    char ID[4];
    unsigned char Length;
};

struct DFFChunkLoudspeaker {
    unsigned short Configuration;
};

struct DFFLoudspeakerChannel {
    char Channel[4];
};

#pragma pack (pop)


int main(int argc, char *argv[])
{
    setlocale(LC_ALL, "");

    const char AppName[]={"dsfdff"};
    const char AppVersion[]={"1.0"};

    char message[1024];

    bool EndOfSourceFile=false;
    bool SourceFileError=false;
    int64_t TotalSourceOffset=0;//bytes

    ChunkHeader header;

    bool DFFIsDetected=false;
    bool compressed=false;

    uint32_t samplingrate=0;
    short Channels=0;

    DSFHeader DSFFileHeader;
    uint32_t DSFChannelConfiguration=0;
    bool DSFReverseByte=false;
    SafeArray <char> DSFSampleBuffer;

    SafeArray <DFFLoudspeakerChannel> DFFChannelConfiguration;
    SafeArray <char> DFFSampleBuffer;
    int64_t ActualByteCountRead=0;
    int64_t TotalDFFLength=0;
    int64_t TotalDFFAudioLength=0;
    int64_t DFFPROPPosition=0;
    int64_t DFFPropLength=0;//with chunk ID

    cout << AppName << " free " << AppVersion << endl;

    #ifdef __UTF16_CMDLINE__
    LPWSTR pCmdLine=GetCommandLineW();
    int NumArgs=0;
    LPWSTR * pCmdArgs=CommandLineToArgvW(pCmdLine, &NumArgs);
    #endif

    if(argc == 2) {
        #ifdef __UTF16_CMDLINE__
        int res=wcscmp(pCmdArgs[1], L"--license");
        #else
        int res=strcmp(argv[1], "--license");
        #endif
        if(!res) {
            cout << LicenseText << endl;
            return 0;
            }
        }

    cout << "Read license: " << AppName << " --license" << endl;

    #ifdef _WIN32
    cout << "Example: " << AppName << " \"sourcefile.dsf\" \"targetfile.dff\"" << endl;
    cout << "         " << AppName << " \"sourcefile.dff\" \"targetfile.dsf\"" << endl;
    #else
    cout << "Example: " << AppName << " 'sourcefile.dsf' 'targetfile.dff'" << endl;
    cout << "         " << AppName << " 'sourcefile.dff' 'targetfile.dsf'" << endl;
    #endif

    if(argc < 3) {
        cout << "Lack of parameters. See example." << endl;
        return -1;
    }
    #ifdef _WIN32
    #ifdef __UTF16_CMDLINE__
    IntString SourceFile=pCmdArgs[1];
    #else
    IntString SourceFile=StrToWCHAR(argv[1]);
    #endif
    if(SourceFile==IntString(L"")) {
        cout << "error: source file name recode" << endl;
    }

    #ifdef __UTF16_CMDLINE__
    IntString TargetFile=pCmdArgs[2];
    #else
    IntString TargetFile=StrToWCHAR(argv[2]);
    #endif
    if(TargetFile==IntString(L"")) {
        cout << "error: target file name recode" << endl;
    }

    #else
    string TempStr=argv[1];
    IntString SourceFile(TempStr.begin(), TempStr.end());
    TempStr=argv[2];
    IntString TargetFile(TempStr.begin(), TempStr.end());
    #endif

    /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
    #ifdef _WIN32
    bool IsDSFSource= ( (ExtractFileExtension(SourceFile)==IntString(L".dsf")) || (ExtractFileExtension(SourceFile)==IntString(L".DSF")) );
    bool IsDFFSource= ( (ExtractFileExtension(SourceFile)==IntString(L".dff")) || (ExtractFileExtension(SourceFile)==IntString(L".DFF")) );

    bool IsDSFTarget=( (ExtractFileExtension(TargetFile)==IntString(L".dsf"))|| (ExtractFileExtension(TargetFile)==IntString(L".DSF")) );
    bool IsDFFTarget=( (ExtractFileExtension(TargetFile)==IntString(L".dff"))|| (ExtractFileExtension(TargetFile)==IntString(L".DFF")) );
    #else
    bool IsDSFSource= ( (ExtractFileExtension(SourceFile)==IntString(".dsf")) || (ExtractFileExtension(SourceFile)==IntString(".DSF")) );
    bool IsDFFSource= ( (ExtractFileExtension(SourceFile)==IntString(".dff")) || (ExtractFileExtension(SourceFile)==IntString(".DFF")) );

    bool IsDSFTarget=( (ExtractFileExtension(TargetFile)==IntString(".dsf"))|| (ExtractFileExtension(TargetFile)==IntString(".DSF")) );
    bool IsDFFTarget=( (ExtractFileExtension(TargetFile)==IntString(".dff"))|| (ExtractFileExtension(TargetFile)==IntString(".DFF")) );
    #endif

    if( !( (IsDFFSource && IsDSFTarget)||(IsDSFSource && IsDFFTarget) ) ) {
        cout << "error: unsupported file" << endl;
        return -1;
    }

    /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

    FileHandle SourceHandle=OpenFile(SourceFile, true);
    if(!SourceHandle) {
        cout << "error: source file open" << endl;
        return -1;
    }

    FileHandle TargetHandle=OpenFile(TargetFile, false);
    if(!TargetHandle) {
        cout << "error: target file open" << endl;
        CloseFile(SourceHandle);
        return -1;
    }


    if(IsDSFSource)
        {//read dsf
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        size_t DSFHeaderSize=sizeof(DSFFileHeader);
        size_t ReadLength=ReadFile(SourceHandle, (char *)&DSFFileHeader, DSFHeaderSize);

        TotalSourceOffset+=ReadLength;
        if(ReadLength!=DSFHeaderSize) {
            cout << "error: source file header read" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }

        samplingrate=DSFFileHeader.SamplingRate;
        cout << "DSF sampling rate: " << samplingrate << endl;

        DSFReverseByte=(DSFFileHeader.BitOrder==8);
        cout << "DSF bit order: " << DSFReverseByte << endl;

        Channels=DSFFileHeader.ChannelNumber;
        cout << "DSF channels: " << Channels << endl;

        DSFChannelConfiguration=DSFFileHeader.ChannelType;
        cout << "DSF channel config: " << DSFChannelConfiguration << endl;

        cout << "DSF sample count: " << DSFFileHeader.SampleCount << endl;

        cout << "DSF data length (bytes): " << DSFFileHeader.DataLength-12 << endl;

        size_t LengthForWrite=0;
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        header.ID[0]='F';header.ID[1]='R';header.ID[2]='M';header.ID[3]='8';
        header.Length=sizeof(DFFChunkFRM8);SwapLongLongWord(&header.Length);

        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target chunk frm8 writing" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;

        DFFChunkFRM8 FRM8Data;
        FRM8Data.DSD_ID[0]='D';FRM8Data.DSD_ID[1]='S';FRM8Data.DSD_ID[2]='D';FRM8Data.DSD_ID[3]=' ';
        LengthForWrite=sizeof(FRM8Data);
        if(WriteFile(TargetHandle, (char *)&FRM8Data, LengthForWrite) != LengthForWrite) {
            cout << "error: target FRM8 data write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;

        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        header.ID[0]='F';header.ID[1]='V';header.ID[2]='E';header.ID[3]='R';
        header.Length=sizeof(DFFChunkVersion);SwapLongLongWord(&header.Length);
        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target FRM8 write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;

        DFFChunkVersion VerData;
        VerData.Version=0x01050000;SwapLongWord(&VerData.Version);
        LengthForWrite=sizeof(VerData);
        if(WriteFile(TargetHandle, (char *)&VerData, LengthForWrite) != LengthForWrite) {
            cout << "error: target FRM8 data write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;

        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        DFFPROPPosition=TotalDFFLength;
        DFFPropLength=0;
        header.ID[0]='P';header.ID[1]='R';header.ID[2]='O';header.ID[3]='P';
        header.Length=0;
        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target FRM8 write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        const char DFFPROP_SNDID[4]={'S','N','D',' '};
        LengthForWrite=sizeof(DFFPROP_SNDID);
        if(WriteFile(TargetHandle, (char *)DFFPROP_SNDID, LengthForWrite) != LengthForWrite) {
            cout << "error: target PROP SND ID write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        /* sample rate */
        header.ID[0]='F';header.ID[1]='S';header.ID[2]=' ';header.ID[3]=' ';
        header.Length=sizeof(DFFChunkSamplingRate);SwapLongLongWord(&header.Length);
        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target FS write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        DFFChunkSamplingRate SamplingRateData;
        SamplingRateData.SamplingRate=samplingrate;SwapLongWord(&SamplingRateData.SamplingRate);
        LengthForWrite=sizeof(SamplingRateData);
        if(WriteFile(TargetHandle, (char *)&SamplingRateData, LengthForWrite) != LengthForWrite) {
            cout << "error: target FS data write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        /* channels */
        SafeArray <char> ChannelConfigurationPtr;
        ChannelConfigurationPtr.AssignLength(Channels*4);
        memset(ChannelConfigurationPtr.data(), 0x20, ChannelConfigurationPtr.length());

        char * ChannelConfugurationStr=ChannelConfigurationPtr.data();
        int ChannelConfigurationStrLength=0;

        switch(DSFChannelConfiguration) {
            case 1:
                memcpy(ChannelConfugurationStr+0, "SLFT", 4);
                ChannelConfigurationStrLength=1;
                break;
            case 2:
                memcpy(ChannelConfugurationStr, "SLFT", 4);
                memcpy(ChannelConfugurationStr+4, "SRGT", 4);
                ChannelConfigurationStrLength=2;
                break;
            case 3:
                memcpy(ChannelConfugurationStr, "MLFT", 4);
                memcpy(ChannelConfugurationStr+4, "MRGT", 4);
                memcpy(ChannelConfugurationStr+4*2, "C   ", 4);
                ChannelConfigurationStrLength=3;
                break;
            case 4:
                memcpy(ChannelConfugurationStr, "MLFT", 4);
                memcpy(ChannelConfugurationStr+4, "MRGT", 4);
                memcpy(ChannelConfugurationStr+4*2, "LS  ", 4);
                memcpy(ChannelConfugurationStr+4*3, "RS  ", 4);
                ChannelConfigurationStrLength=4;
                break;
            case 5:
                memcpy(ChannelConfugurationStr, "MLFT", 4);
                memcpy(ChannelConfugurationStr+4, "MRGT", 4);
                memcpy(ChannelConfugurationStr+4*2, "LS  ", 4);
                memcpy(ChannelConfugurationStr+4*3, "RS  ", 4);
                ChannelConfigurationStrLength=4;
                break;
            case 6:
                memcpy(ChannelConfugurationStr, "MLFT", 4);
                memcpy(ChannelConfugurationStr+4, "MRGT", 4);
                memcpy(ChannelConfugurationStr+4*2, "C   ", 4);
                memcpy(ChannelConfugurationStr+4*3, "LS  ", 4);
                memcpy(ChannelConfugurationStr+4*4, "RS  ", 4);
                ChannelConfigurationStrLength=5;
                break;
            case 7:
                memcpy(ChannelConfugurationStr, "MLFT", 4);
                memcpy(ChannelConfugurationStr+4, "MRGT", 4);
                memcpy(ChannelConfugurationStr+4*2, "C   ", 4);
                memcpy(ChannelConfugurationStr+4*3, "LFE ", 4);
                memcpy(ChannelConfugurationStr+4*4, "LS  ", 4);
                memcpy(ChannelConfugurationStr+4*5, "RS  ", 4);
                ChannelConfigurationStrLength=6;
                break;
        }

        if((ChannelConfigurationStrLength != Channels)||(!ChannelConfigurationStrLength)) {
            cout << "error: wrong CHNL configuration" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }

        header.ID[0]='C';header.ID[1]='H';header.ID[2]='N';header.ID[3]='L';
        header.Length=sizeof(DFFChunkChannels)+4*ChannelConfigurationStrLength;
        SwapLongLongWord(&header.Length);
        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target CHNL write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        DFFChunkChannels ChannelsData;
        ChannelsData.ChannelNumber=Channels;SwapShortWord(&ChannelsData.ChannelNumber);
        LengthForWrite=sizeof(ChannelsData);
        if(WriteFile(TargetHandle, (char *)&ChannelsData, LengthForWrite) != LengthForWrite) {
            cout << "error: target CHNL data write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        LengthForWrite=4*ChannelConfigurationStrLength;
        if(WriteFile(TargetHandle, (char *)ChannelConfugurationStr, LengthForWrite) != LengthForWrite) {
            cout << "error: target CHNL configuration write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        /* compression */
        const char NotCompressedString[]={"not compressed"};
        header.ID[0]='C';header.ID[1]='M';header.ID[2]='P';header.ID[3]='R';
        header.Length=sizeof(DFFChunkCompression)+sizeof(NotCompressedString);SwapLongLongWord(&header.Length);
        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target CMPR write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        DFFChunkCompression CompressionData;
        CompressionData.ID[0]='D';CompressionData.ID[1]='S';CompressionData.ID[2]='D';CompressionData.ID[3]=' ';
        CompressionData.Length=sizeof(NotCompressedString)-1;
        LengthForWrite=sizeof(CompressionData);
        if(WriteFile(TargetHandle, (char *)&CompressionData, LengthForWrite) != LengthForWrite) {
            cout << "error: target CMPR data write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        LengthForWrite=sizeof(NotCompressedString);
        if(WriteFile(TargetHandle, (char *)NotCompressedString, LengthForWrite) != LengthForWrite) {
            cout << "error: target CMPR name write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        /* loudspeaker configuration */
        DFFChunkLoudspeaker  DFFLoudspeakerData;
        switch(Channels)
            {
            case 2:
                DFFLoudspeakerData.Configuration=0;
                break;
            case 5:
                DFFLoudspeakerData.Configuration=3;
                break;
            case 6:
                DFFLoudspeakerData.Configuration=4;
                break;
            default:
                DFFLoudspeakerData.Configuration=65535;
            }
        SwapShortWord(&DFFLoudspeakerData.Configuration);



        header.ID[0]='L';header.ID[1]='S';header.ID[2]='C';header.ID[3]='O';
        header.Length=sizeof(DFFLoudspeakerData);SwapLongLongWord(&header.Length);
        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target LSCO write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        LengthForWrite=sizeof(DFFChunkLoudspeaker);
        if(WriteFile(TargetHandle, (char *)&DFFLoudspeakerData, LengthForWrite) != LengthForWrite) {
            cout << "error: target speaker configuration data write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;
        DFFPropLength+=LengthForWrite;

        /* correct prop chunk length */
        FileGotoPosition(TargetHandle, DFFPROPPosition);

        header.ID[0]='P';header.ID[1]='R';header.ID[2]='O';header.ID[3]='P';
        header.Length=DFFPropLength-12;
        SwapLongLongWord(&header.Length);
        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target set PROP write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }

        FileGotoPosition(TargetHandle, DFFPROPPosition+DFFPropLength);

        /* audio */
        int64_t DFFDSDPosition=TotalDFFLength;
        header.ID[0]='D';header.ID[1]='S';header.ID[2]='D';header.ID[3]=' ';
        header.Length=0;
        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target chunk audio header write" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }
        TotalDFFLength+=LengthForWrite;

        size_t DSFReadBlockLengthBytes=DSFHeader::DSFBlockLengthBytes*Channels;
        if(DSFSampleBuffer.length() != DSFReadBlockLengthBytes)
            DSFSampleBuffer.AssignLength(DSFReadBlockLengthBytes);
        if(DFFSampleBuffer.length() != DSFReadBlockLengthBytes)
            DFFSampleBuffer.AssignLength(DSFReadBlockLengthBytes);


        for(int64_t sample=0; sample < (int64_t)DSFFileHeader.SampleCount; sample+=DSFHeader::DSFBlockLengthSamples) {
            //read dsf audio
            size_t LengthForRead=DSFReadBlockLengthBytes;
            size_t ReadLenght=ReadFile(SourceHandle, DSFSampleBuffer.data(), LengthForRead);
            if(!ReadLenght)
                break;

            TotalSourceOffset+=ReadLenght;

            if(ReadLenght != LengthForRead) {
                cout << "warning: source audio damaged (" << TotalSourceOffset << ")" << endl;
                break;
            }

            int64_t DSFRestOfSamples=DSFFileHeader.SampleCount-sample;
            if(DSFRestOfSamples > DSFHeader::DSFBlockLengthSamples)
                DSFRestOfSamples=DSFHeader::DSFBlockLengthSamples;

            for(int64_t chan=0;chan<Channels;chan++) {
                int64_t DSFOffset=chan*DSFHeader::DSFBlockLengthBytes;
                int64_t DFFOffset=chan;
                int64_t byte=0;
                for(;byte<DSFRestOfSamples/8;byte++,DSFOffset++,DFFOffset+=Channels) {
                    char tmp=DSFSampleBuffer.data()[DSFOffset];
                    if(!DSFReverseByte)
                        SwapCharBites(&tmp);
                    DFFSampleBuffer.data()[DFFOffset]=tmp;
                }
            }

            LengthForWrite=DSFRestOfSamples/8*Channels;
            if(WriteFile(TargetHandle, DFFSampleBuffer.data(), LengthForWrite) != LengthForWrite) {
                cout << "error: target audio data write" << endl;
                CloseFile(SourceHandle);
                CloseFile(TargetHandle);
                RemoveFile(TargetFile.c_str());
                return -1;
            }
            TotalDFFLength+=LengthForWrite;
            TotalDFFAudioLength+=LengthForWrite;

            if(DSFRestOfSamples < DSFHeader::DSFBlockLengthSamples) {
                break;
            }
        }

        cout << "DFF audio data size (bytes): " << TotalDFFAudioLength << endl;
        cout << "DFF audio data size (samples): " << (TotalDFFAudioLength/Channels*8) << endl;

        /* correct dsd chunk length */
        FileGotoPosition(TargetHandle, DFFDSDPosition);

        header.ID[0]='D';header.ID[1]='S';header.ID[2]='D';header.ID[3]=' ';
        header.Length=TotalDFFAudioLength;
        SwapLongLongWord(&header.Length);
        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target DFF DSD chunk rewrite" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }

        /* correct frm8 chunk length */
        FileGotoPosition(TargetHandle, 0);

        header.ID[0]='F';header.ID[1]='R';header.ID[2]='M';header.ID[3]='8';
        header.Length=TotalDFFLength-12;
        SwapLongLongWord(&header.Length);
        LengthForWrite=sizeof(header);
        if(WriteFile(TargetHandle, (char *)&header, LengthForWrite) != LengthForWrite) {
            cout << "error: target DFF FRM8 chunk rewrite" << endl;
            CloseFile(SourceHandle);
            CloseFile(TargetHandle);
            RemoveFile(TargetFile.c_str());
            return -1;
        }

        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
    }
    if(IsDFFSource)
        {//read dff
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
        DSFFileHeader.DataLength=0;

        if(WriteFile(TargetHandle, (char *)&DSFFileHeader, sizeof(DSFFileHeader)) != sizeof(DSFFileHeader)) {
            cout << "error: target file write" << endl;
            SourceFileError=true;
            EndOfSourceFile=true;
            }
        DSFFileHeader.FileLength=sizeof(DSFFileHeader);

        while(!EndOfSourceFile)
            {
            /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
            size_t HeaderSize=sizeof(header);
            size_t ReadHeaderLength=ReadFile(SourceHandle, (char *)&header, HeaderSize);
            TotalSourceOffset+=ReadHeaderLength;
            if(ReadHeaderLength!=HeaderSize) {
                EndOfSourceFile=true;
                continue;
            }
            SwapLongLongWord(&header.Length);

            size_t ChunkSize=0;
            size_t ReadDataLen=0;
            /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
            if((header.ID[0]=='F')&&(header.ID[1]=='R')&&(header.ID[2]=='M')&&(header.ID[3]=='8')) {
                DFFIsDetected=true;

                cout << "DFF length (header): " << header.Length << endl;

                DFFChunkFRM8 FRM8Data;
                ChunkSize=sizeof(DFFChunkFRM8);
                ReadDataLen=ReadFile(SourceHandle, (char *)&FRM8Data, ChunkSize);
                if(ReadDataLen!=ChunkSize) {
                    EndOfSourceFile=true; SourceFileError=true;
                    TotalSourceOffset+=ReadDataLen;
                    continue;
                    }
                TotalSourceOffset+=ChunkSize;
                if(!((FRM8Data.DSD_ID[0]=='D')&&(FRM8Data.DSD_ID[1]=='S')&&(FRM8Data.DSD_ID[2]=='D')&&(FRM8Data.DSD_ID[3]==' '))) {
                    EndOfSourceFile=true; SourceFileError=true;
                    cout << "no FRM8 id" << endl;
                    }

                continue;
            }
            if(!DFFIsDetected) {
                SourceFileError=true;
                EndOfSourceFile=true;
                continue;
            }

            /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
            if((header.ID[0]=='F')&&(header.ID[1]=='V')&&(header.ID[2]=='E')&&(header.ID[3]=='R')) {
                DFFChunkVersion DFFVersion;
                ChunkSize=sizeof(DFFChunkVersion);
                ReadDataLen=ReadFile(SourceHandle, (char *)&DFFVersion, ChunkSize);
                if(ReadDataLen!=ChunkSize) {
                    TotalSourceOffset+=ReadDataLen;
                    SourceFileError=true;
                    EndOfSourceFile=true;
                    cout << "error: invalid DFF version" << endl;
                    continue;
                    }
                TotalSourceOffset+=ChunkSize;
                SwapLongWord(&DFFVersion.Version);
                sprintf(message, "%08x", DFFVersion.Version);
                cout << "DFF version: " << message << endl;
                continue;
            }

            /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
            if((header.ID[0]=='P')&&(header.ID[1]=='R')&&(header.ID[2]=='O')&&(header.ID[3]=='P')) {
                uint64_t DFFPROPLength=header.Length;
                char DFFPROP_SND_ID[4];
                ChunkSize=sizeof(DFFPROP_SND_ID);
                if(ChunkSize > DFFPROPLength) {
                    if(DFFPROPLength > 0)
                        FileGotoFromCurrent(SourceHandle, DFFPROPLength);
                    cout << "error: SND ID" << endl;
                    continue;
                    }
                ReadDataLen=ReadFile(SourceHandle, DFFPROP_SND_ID, ChunkSize);
                TotalSourceOffset+=ReadDataLen; DFFPROPLength-=ReadDataLen;
                if(ReadDataLen!=ChunkSize) {
                    EndOfSourceFile=true;
                    SourceFileError=true;
                    cout << "error: invalid SND ID" << endl;
                    continue;
                    }
                if(!((DFFPROP_SND_ID[0]=='S')&&(DFFPROP_SND_ID[1]=='N')&&(DFFPROP_SND_ID[2]=='D')&&(DFFPROP_SND_ID[3]==' '))) {
                    cout << "error: invalid SND ID content" << endl;
                    continue;
                }

                /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

                if(DFFPROPLength == 0) {
                    cout << "error: truncated PROP chunk" << endl;
                    continue;
                }

                SafeArray <char> DFFPropData;
                DFFPropData.AssignLength(DFFPROPLength);

                ChunkSize=DFFPROPLength;
                ReadDataLen=ReadFile(SourceHandle, DFFPropData.data(), ChunkSize);
                TotalSourceOffset+=ReadDataLen;
                if(ReadDataLen!=ChunkSize) {
                    EndOfSourceFile=true; SourceFileError=true;
                    cout << "error: PROP chunk data" << endl;
                    continue;
                }

               int64_t pos=0, reserve=DFFPROPLength;
               while(pos < (int64_t)(DFFPROPLength-sizeof(ChunkHeader))) {
                   ChunkHeader * HeaderPtr=(ChunkHeader *)(DFFPropData.data()+pos);
                   SwapLongLongWord(&(HeaderPtr->Length));

                   int64_t ChunkDataLength=HeaderPtr->Length;
                   pos+=sizeof(ChunkHeader); reserve-=sizeof(ChunkHeader);
                   if((reserve <= 0)||(ChunkDataLength <= 0)||(ChunkDataLength > reserve)) {
                       cout << "invalid data" << endl; break;}

                   if((HeaderPtr->ID[0]=='F')&&(HeaderPtr->ID[1]=='S')&&(HeaderPtr->ID[2]==' ')&&(HeaderPtr->ID[3]==' ')) {
                       if(ChunkDataLength != sizeof(DFFChunkSamplingRate)) {
                           cout << "invalid data: FS" << endl; break; }

                       DFFChunkSamplingRate * SamplingRatePtr=(DFFChunkSamplingRate *)(DFFPropData.data()+pos);
                       SwapLongWord(&(SamplingRatePtr->SamplingRate)); samplingrate=SamplingRatePtr->SamplingRate;

                       pos+=ChunkDataLength; reserve-=ChunkDataLength;
                       continue;
                       }

                   if((HeaderPtr->ID[0]=='C')&&(HeaderPtr->ID[1]=='H')&&(HeaderPtr->ID[2]=='N')&&(HeaderPtr->ID[3]=='L')) {
                       if(ChunkDataLength <= (int64_t)sizeof(DFFChunkChannels)) {
                           cout << "invalid data: CHNL" << endl; break; }

                       DFFChunkChannels * ChannelPtr=(DFFChunkChannels *)(DFFPropData.data()+pos);
                       SwapShortWord(&(ChannelPtr->ChannelNumber)); Channels=ChannelPtr->ChannelNumber;
                       pos+=sizeof(DFFChunkChannels); reserve-=sizeof(DFFChunkChannels); ChunkDataLength-=sizeof(DFFChunkChannels);

                       if(Channels <= 0) {
                           cout << "invalid data: CHNL number" << endl; break; }

                       int64_t ChannelConfigurationLength=sizeof(DFFLoudspeakerChannel)*Channels;
                       if(ChannelConfigurationLength != ChunkDataLength) {
                           cout << "invalid data: CHNL data" << endl; break; }
                       DFFChannelConfiguration.AssignLength(Channels);

                       DFFLoudspeakerChannel * TargetChannelPtr=DFFChannelConfiguration.data();
                       DFFLoudspeakerChannel * SourceChannelPtr=(DFFLoudspeakerChannel *)(DFFPropData.data()+pos);;

                       for(int i=0; i < Channels; i++) {
                           TargetChannelPtr[i]=SourceChannelPtr[i];
                           cout << "DFF CHNL: " << SourceChannelPtr[i].Channel[0] << SourceChannelPtr[i].Channel[1] << SourceChannelPtr[i].Channel[2] << SourceChannelPtr[i].Channel[3] << endl;
                           }

                       pos+=ChunkDataLength; reserve-=ChunkDataLength;
                       continue;
                   }

                   if((HeaderPtr->ID[0]=='C')&&(HeaderPtr->ID[1]=='M')&&(HeaderPtr->ID[2]=='P')&&(HeaderPtr->ID[3]=='R')) {
                       if(ChunkDataLength <= (int64_t)sizeof(DFFChunkCompression)) {
                           cout << "invalid data: CMPR" << endl; break; }
                       DFFChunkCompression * CompressionPtr=(DFFChunkCompression *)(DFFPropData.data()+pos);

                       compressed=!((CompressionPtr->ID[0]=='D')&&(CompressionPtr->ID[1]=='S')&&(CompressionPtr->ID[2]=='D')&&(CompressionPtr->ID[3]==' '));

                       pos+=sizeof(DFFChunkChannels); reserve-=sizeof(DFFChunkChannels); ChunkDataLength-=sizeof(DFFChunkChannels);
                       pos+=ChunkDataLength; reserve-=ChunkDataLength;
                       continue;
                   }


                   pos+=ChunkDataLength; reserve-=ChunkDataLength;
                }
                continue;
            }

            /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
            if(((header.ID[0]=='D')&&(header.ID[1]=='S')&&(header.ID[2]=='T')&&(header.ID[3]==' '))||compressed) {
                cout << "error: input file is compressed" << endl;
                SourceFileError=true;
                compressed=true;
                break;
            }

            /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
            if((header.ID[0]=='D')&&(header.ID[1]=='S')&&(header.ID[2]=='D')&&(header.ID[3]==' ')) {
                if(Channels <= 0) {
                    SourceFileError=true;
                    cout << "error: invalid channel number" << endl;
                    break;
                }
                if(!samplingrate) {
                    SourceFileError=true;
                    cout << "error: invalid sampling rate" << endl;
                    break;
                }

                DSFFileHeader.SamplingRate=samplingrate;
                DSFFileHeader.BitOrder=1;
                DSFFileHeader.ChannelNumber=Channels;
                DSFFileHeader.ChannelType=0;

                switch (Channels) {
                    case 1:
                        DSFFileHeader.ChannelType=1;
                        break;
                    case 2:
                        DSFFileHeader.ChannelType=2;
                        break;
                    case 3:
                        DSFFileHeader.ChannelType=3;
                        break;
                    case 4:
                        DSFFileHeader.ChannelType=4;
                        break;
                    case 5:
                        DSFFileHeader.ChannelType=6;
                        break;
                    case 6:
                        DSFFileHeader.ChannelType=7;
                        break;
                    default:
                        break;
                }

                int64_t DSFBlockForReadLength=DSFFileHeader.BlockSizePerChannel*Channels;
                int64_t DSFBlockLengthActual=DSFFileHeader.BlockSizePerChannel;
                int32_t CurrentChan, CurrentByte, DSFOffset, DFFOffset;

                if((int64_t)DSFSampleBuffer.length() != DSFBlockForReadLength)
                    DSFSampleBuffer.AssignLength(DSFBlockForReadLength);
                if((int64_t)DFFSampleBuffer.length() != DSFBlockForReadLength)
                    DFFSampleBuffer.AssignLength(DSFBlockForReadLength);

                DSFFileHeader.SampleCount=0;

                cout << "DFF audio data length (samples)" << header.Length*8/Channels << endl;

                for(int64_t DFFCurrentByte=0; DFFCurrentByte < (int64_t)header.Length;) {
                    int64_t DFFRestOfAudio=header.Length-DFFCurrentByte;
                    int64_t LengthForRead=DFFSampleBuffer.length();
                    if(LengthForRead > DFFRestOfAudio)
                        LengthForRead=DFFRestOfAudio;

                    size_t DFFSourceReadLenghtActual=ReadFile(SourceHandle, DFFSampleBuffer.data(), LengthForRead);
                    TotalSourceOffset+=DFFSourceReadLenghtActual;
                    DFFCurrentByte+=DFFSourceReadLenghtActual; ActualByteCountRead+=DFFSourceReadLenghtActual;
                    DSFBlockLengthActual=DFFSourceReadLenghtActual/Channels;

                    if(DSFBlockLengthActual) {
                        for(CurrentChan=0; CurrentChan<Channels;CurrentChan++) {
                            CurrentByte=0;
                            DSFOffset=CurrentChan*DSFFileHeader.BlockSizePerChannel;
                            DFFOffset=CurrentChan;
                            for(;CurrentByte<DSFBlockLengthActual;CurrentByte++, DSFOffset++, DFFOffset+=Channels) {
                                SwapCharBites(DFFSampleBuffer.data()+DFFOffset);
                                DSFSampleBuffer.data()[DSFOffset]=DFFSampleBuffer.data()[DFFOffset];
                            }
                        }
                        if(DSFBlockLengthActual < DSFFileHeader.BlockSizePerChannel) {
                            for(CurrentChan=0; CurrentChan<Channels;CurrentChan++) {
                                CurrentByte=DSFBlockLengthActual;
                                DSFOffset=CurrentChan*DSFFileHeader.BlockSizePerChannel+DSFBlockLengthActual;
                                for(; CurrentByte<(int32_t)DSFFileHeader.BlockSizePerChannel;CurrentByte++, DSFOffset++)
                                    DSFSampleBuffer.data()[DSFOffset]=0;
                            }
                        }
                        DSFFileHeader.SampleCount+=DSFBlockLengthActual*8;

                        uint64_t DSFLengthForWrite=DSFFileHeader.BlockSizePerChannel*Channels;
                        DSFFileHeader.DataLength+=DSFLengthForWrite;
                        DSFFileHeader.FileLength+=DSFLengthForWrite;

                        uint64_t written=WriteFile(TargetHandle, DSFSampleBuffer.data(), DSFLengthForWrite);
                        if(written != DSFLengthForWrite) {
                            SourceFileError=true;
                            cout << "error: target file audio write" << endl;
                            break;
                        }
                    }
                else
                    break;

                }
                continue;
            }

            FileGotoFromCurrent(SourceHandle, header.Length);
            TotalSourceOffset+=header.Length;
            int64_t CurrentFilePos=FileCurrentPosition(SourceHandle);
            if(CurrentFilePos > 0)
                if(TotalSourceOffset > CurrentFilePos) TotalSourceOffset=CurrentFilePos;
            }

        cout << "DSF audio data length (samples)" << DSFFileHeader.SampleCount << endl;


        FileGotoPosition(TargetHandle, 0);
        DSFFileHeader.DataLength+=12;
        if(WriteFile(TargetHandle, (char *)&DSFFileHeader, sizeof(DSFFileHeader)) != sizeof(DSFFileHeader)) {
            cout << "error: target file header rewrite" << endl;
            SourceFileError=true;
            EndOfSourceFile=true;
        }
    }


    CloseFile(TargetHandle);
    CloseFile(SourceHandle);

    if(SourceFileError)
        RemoveFile(TargetFile);

    cout << "Action complete" << endl;

    return 0;
}
